@@ -40,6 +40,14 @@
 //     Ok(())
 // }
 
+// use bioutils::utils::check::value::Check;
+use bioutils::utils::check::value::CheckU8;
+use bioutils::utils::get::item::GetItemU8;
 fn main(){
-    
-}
\ No newline at end of file
+    let test = b"ACTG";
+    test.cut_to_length(&6);
+    test.is_homopolymer();
+}
+
+// Get the sums of each flag in a bam file
+
