@@ -1,44 +1,14 @@
-
-//! Trait for checking specific criteria for a u8 of biological file origin. Types include sequence (nucleotide/amino acid) and quality (phred33/64/solexa, phred33 being all printable ascii).
-//! See below examples for included functions. Note that quality alphabets overlap, and one quality line may be valid phred33,64, solexa, or any combination. Check with the manufacturer for which quality encoding your data has. If unsure, most data generated beyond 2016 is kept in phred33 encoding.
-//! Additional functionality for common checks including has_n, has_gap, is_homopolymer, etc.
-//! # Examples
-//! ```
-//! use crate::bioutils::charsets;
-//! use crate::bioutils::utils::check::value::CheckU8;
-//! let dna = b"ACTG";
-//! let rna = b"ACUG";
-//! let homopolymerN = b"NNNN";
-//! let homopolymerA = b"AAAA";
-//! let gapna = b"AC-G";
-//! let nna = b"ACnG";
-//! let quality = b"@ABC";
-//!
-//! assert!(homopolymerN.is_homopolymer());
-//! assert!(homopolymerA.is_homopolymer_not_n());
-//! assert!(homopolymerN.is_homopolymer_n());
-//!
-//! assert!(gapna.has_gap());
-//! assert!(nna.has_n());
-//! assert!(dna.is_iupac());
-//! assert!(rna.is_basic_rna());
-//!
-//! assert!(quality.is_phred33());
-//! assert!(quality.is_phred64());
-//! assert!(quality.is_solexa());
-//! 
-//! ```
-
-// use crate::utils::check_percentage_u8;
 use std::borrow::Borrow;
 use crate::utils::get::value::percentage;
-use crate::utils::get::value::validate_percentage_u8;
 use std::collections::HashMap;
 use crate::charsets::PERCENTAGE_RANGE;
 
 use crate::charsets::ascii::*;
 use crate::utils::get::value;
 use crate::utils::get::value::ValueU8;
+use crate::utils::get::value::AsMutValueU8;
+use crate::utils::get::item::CodeItemU8;
+use crate::utils::mutate::item::MutCodeItemU8;
 
 /// Trait for checking specific criteria for a u8 of biological file origin. Types include sequence (nucleotide/amino acid) and quality (phred33/64/solexa, phred33 being all printable ascii).
 /// These should be used with closely with the is_ascii/make/to_ascii_lowercase/make/to_ascii_uppercase functions in standard rust.
@@ -73,17 +43,17 @@ where
 {
         /// Checks if the sequence or quality u8 is less than or equal to the given length. Used to cut read to minimum length.
         fn is_at_least_length(&self, length: &usize) -> bool {
-            self.as_ref().iter().len() >= *length
+            self.as_ref().len() >= *length
         }
     
         /// Checks if the sequence or quality u8 is greater than or equal to the given length. Used to cut read to maximum length.
         fn is_at_most_length(&self, length: &usize) -> bool {
-            self.as_ref().iter().len() <= *length
+            self.as_ref().len() <= *length
         }
     
         /// Checks if equal to the given length.
         fn is_length(&self, length: &usize) -> bool {
-            self.as_ref().iter().len() == *length
+            self.as_ref().len() == *length
         }
 
         /// Checks if completely comprised of the same character. Does not use a character set, so could be all gaps, etc. Use has_mixed_case and to_uppercase/to_lowercase prior if mixed case.
@@ -101,49 +71,13 @@ where
         }
 }
 
-    // /// Validates whether is a valid something based on the boolean is_x smaller functions in this trait and returns a wrapped boolean. Example: check_u8(b"ACTG","is_basic_dna") returns a wrapped "true". Options for is_what are the names of the charset boolean functions:
-    // /// is_basic_dna, is_phred33, is_basic_rna,  
-    // /// The goal of this function would be to set is_what to a constant in the program, for example a program focused on illumina data might set a constant to phred33 and input as is_what, rather than having to call is_phred33 each time. This means we can easily make our program take phred33 or 64 by just changing the constant.
-    // fn check_u8(&self, is_what: &str) -> Result<bool, &str>;
-
-// /// Validates whether is a valid something based on the boolean is_x smaller functions in this trait and returns a wrapped boolean. Example: check_u8(b"ACTG","is_basic_dna") returns a wrapped "true". Options for is_what are the names of the charset boolean functions:
-    // /// "is_iupac_nucleotide", "is_iupac_amino_acid", "is_iupac",
-    // /// "is_phred33", "is_phred64", "is_solexa",  
-    // /// "is_basic_dna", "is_basic_rna", "is_basic_amino_acid",
-    // /// "is_homopolymer", "is_homopolymer_n", "is_homopolymer_not_n",
-    // /// "has_n", "has_gap",
-    // /// "is_ascii_letters", "is_ascii_letters_uppercase", "is_ascii_letters_lowercase" 
-    // /// The goal of this function would be to set is_what to a constant in the program, for example a program focused on dna data might set a constant to is_basic_dna and input as is_what rather than having to call is_basic_dna each time. Later, if we want to focus on rna, we can easily change our constant to is_basic_rna just by changing the constant.
-    // fn check_u8(&self, is_what: &str) -> Result<bool, &str>{
-    //     if validate_is_what(&is_what).unwrap() {
-    //         match is_what {
-    //             "is_phred33" => Ok(self.is_phred33()),
-    //             "is_phred64" => Ok(self.is_phred64()),
-    //             "is_solexa" => Ok(self.is_solexa()),
-    //             "is_iupac_nucleotide" => Ok(self.is_iupac_nucleotide()),
-    //             "is_iupac_amino_acid" => Ok(self.is_iupac_amino_acid()),
-    //             "is_iupac" => Ok(self.is_iupac()),
-    //             "is_basic_dna" => Ok(self.is_basic_dna()),
-    //             "is_basic_rna" => Ok(self.is_basic_rna()),
-    //             "is_basic_amino_acid" => Ok(self.is_basic_amino_acid()),
-    //             "is_homopolymer" => Ok(CheckU8::is_homopolymer(&self)),
-    //             "is_homopolymer_n" => Ok(self.is_homopolymer_n()),
-    //             "is_homopolymer_not_n" => Ok(self.is_homopolymer_not_n()),
-    //             "has_n" => Ok(self.has_n()),
-    //             "has_gap" => Ok(self.has_gap()),
-    //             "is_ascii_letters" => Ok(self.is_ascii_letters()),
-    //             "is_ascii_letters_uppercase" => Ok(self.is_ascii_letters_uppercase()),
-    //             "is_ascii_letters_lowercase" => Ok(self.is_ascii_letters_lowercase()),
-    //             _ => Err("Invalid is_what parameter, please choose a valid option")
-    //         }
-    //     } else {validate_is_what(&is_what)}
-    // }
-
-
-
 pub trait CheckU8<T> {
     /// Checks the sequence has the percent bases (rounded) above the quality score
-    fn is_qual_passing(&self, quality_score: &u8, percent: &u8) -> Result<bool, &str>;
+    fn is_qual_passing_percent(&self, quality_score: &u8, percent: &u8) -> Result<bool, &str>;
+    /// Checks the encoded sequence has a quality score above greater than or equal to the supplied mean. Decodes from raw read from fastq file with phred33 encoding. Commonly done per base in fastqc.
+    fn is_qual_passing_mean(&self, mean_quality_score: &u8) -> Result<bool, &str>;
+    /// Checks the encoded sequence has a quality score above greater than or equal to the supplied mean. Decodes from raw read from fastq file with phred64 encoding. Commonly done per base in fastqc.
+    fn is_qual_passing_mean_phred64(&self, mean_quality_score: &u8) -> Result<bool, &str>;
 
     /// Checks if the sequence is a homopolymer with percentage cutoff.
     fn is_percent_homopolymer(&self, percent: &u8) -> Result<bool, &str>;
@@ -195,7 +129,7 @@ where
     T: AsRef<[u8]>,
 {
     /// Checks the sequence has a number of bases (percent rounded) greater than or equal to the supplied quality score
-    fn is_qual_passing(&self, quality_score: &u8, percent: &u8) -> Result<bool, &str> {
+    fn is_qual_passing_percent(&self, quality_score: &u8, percent: &u8) -> Result<bool, &str> {
         if validate_percentage_u8(percent).unwrap() {
             if self.quality_percent_passing(&quality_score) >= (*percent).into() {
                 Ok(true)
@@ -203,18 +137,37 @@ where
         } else { validate_percentage_u8(percent) }
     }
 
+    /// Checks the encoded sequence has a quality score above greater than or equal to the supplied mean. Decodes from raw read from fastq file with phred33 encoding. Commonly done per base in fastqc.
+    fn is_qual_passing_mean(&self, mean_quality_score: &u8) -> Result<bool, &str> {
+        if validate_phred33_score_u8(mean_quality_score).unwrap() {
+            if self.decode_qual().mean() >= (*mean_quality_score).into() {
+                Ok(true)
+            } else { Ok(false) }
+        } else { validate_phred33_score_u8(mean_quality_score) }
+    }
+    
+    /// Checks the encoded sequence has a quality score above greater than or equal to the supplied mean. Decodes from raw read from fastq file with phred64 encoding. Commonly done per base in fastqc.
+    fn is_qual_passing_mean_phred64(&self, mean_quality_score: &u8) -> Result<bool, &str> {
+        if validate_phred64_score_u8(mean_quality_score).unwrap() {
+            if self.decode_qual_phred64().mean() >= (*mean_quality_score).into() {
+                Ok(true)
+            } else { Ok(false) }
+        } else { validate_phred64_score_u8(mean_quality_score) }
+    }
+
     /// Checks if the sequence is a homopolymer with percentage cutoff
     fn is_percent_homopolymer(&self, percent: &u8) -> Result<bool, &str> {
         if validate_percentage_u8(&percent).unwrap() {
-            if percentage(self.count_mode(), self.as_ref().iter().len()) >= (*percent).into() {
+            if percentage(self.count_mode(), self.as_ref().len()) >= (*percent).into() {
                 Ok(true)
             } else {Ok(false)}
         } else {validate_percentage_u8(&percent)}
     }
+
     /// Checks if the sequence is comprised of 'x' base greater than 'percent' cutoff. Primary use is for filtering for reads with >90% percent N's or A's
     fn is_percent_homopolymer_x(&self, x: &u8, percent: &u8) -> Result<bool, &str> {
         if validate_percentage_u8(&percent).unwrap() {
-            if percentage(self.count_xu8(x), self.as_ref().iter().len()) >= (*percent).into() {
+            if percentage(self.count_xu8(x), self.as_ref().len()) >= (*percent).into() {
                 Ok(true)
             } else {Ok(false)}
         } else {validate_percentage_u8(&percent)}
@@ -313,9 +266,28 @@ where
 }
 
 
+pub trait MutCheckU8<T> {
+    /// Checks the sequence has a quality score above greater than or equal to the supplied mean.
+    fn is_qual_passing_mean(&mut self, mean_quality_score: &u8) -> Result<bool, &str>;
+}
+
+impl<T> MutCheckU8<T> for T
+where
+    T: AsMut<[u8]>,
+{
+    /// Checks the sequence has a quality score above greater than or equal to the supplied mean. Commonly done per base in fastqc.
+    fn is_qual_passing_mean(&mut self, mean_quality_score: &u8) -> Result<bool, &str> {
+        if validate_phred33_score_u8(mean_quality_score).unwrap() {
+            if self.mut_decode_qual().mut_mean() >= (*mean_quality_score).into() {
+                Ok(true)
+            } else { Ok(false) }
+        } else { validate_phred33_score_u8(mean_quality_score) }
+    }
+}
+
 pub trait CheckEqItems<K>{
     /// Checks if the sequence and quality u8 vectors are the same length. Generally checks two u8 items for length against each other
-    fn is_seq_qual_length_equal(&self, quality: &K) -> bool;
+    fn is_seq_qual_length_equal(&self, comparison: &K) -> bool;
 }
 
 impl<T, K> CheckEqItems<K> for T
@@ -326,11 +298,118 @@ where
     K: PartialEq,
 {
     /// Checks if two items are the same length. 
-    fn is_seq_qual_length_equal(&self, quality: &K)-> bool {
-        self.as_ref().iter().len() == quality.as_ref().iter().len()
+    fn is_seq_qual_length_equal(&self, comparison: &K)-> bool {
+        self.as_ref().len() == comparison.as_ref().len()
     }
 }
 
+
+pub trait CheckPalindrome<T> {
+    /// Generic trait to check if T is a palindrome
+    fn is_palindrome(&self) -> bool;
+}
+
+impl<T> CheckPalindrome<T> for T
+where
+    T: IntoIterator,
+    T::Item: PartialEq,
+    T::IntoIter: DoubleEndedIterator,
+    T: Copy,
+{
+    /// Generic impl to check if T is a palindrome
+    fn is_palindrome(&self) -> bool {
+    let mut iter = self.into_iter();
+    while let (Some(front), Some(back)) = (iter.next(), iter.next_back()) {
+        if front != back {
+            return false;
+        }
+    }
+    true
+    }
+}
+
+
+/// Generic function to check if T is a palindrome.
+pub fn is_palindrome<T>(iterable: T) -> bool
+where
+    T: IntoIterator,
+    T::Item: PartialEq,
+    T::IntoIter: DoubleEndedIterator,
+{
+    let mut iter = iterable.into_iter();
+    while let (Some(forward), Some(backward)) = (iter.next(), iter.next_back()) {
+        if forward != backward {
+            return false;
+        }
+    }
+    true
+}
+
+
+/// Validate a u8 is 0 to 100 and return a wrapped boolean
+pub fn validate_percentage_u8(percent: &u8) -> Result<bool, &'static str> {
+    match PERCENTAGE_RANGE.contains(percent){    
+    true => Ok(true),
+    false => Err("Please supply a percent (0-100, not fractional) as u8"),
+    }
+}
+
+/// Validate a u8 is phred33 score 0-42
+pub fn validate_phred33_score_u8(quality_score: &u8) -> Result<bool, &'static str> {
+    match PHRED33_SCORES_U8.contains(quality_score){    
+    true => Ok(true),
+    false => Err("Please supply a quality score (0-42, not fractional) as u8"),
+    }
+}
+
+/// Validate a u8 is phred64 score 0-42
+pub fn validate_phred64_score_u8(quality_score: &u8) -> Result<bool, &'static str> {
+    match PHRED64_SCORES_U8.contains(quality_score){    
+    true => Ok(true),
+    false => Err("Please supply a quality score (0-42, not fractional) as u8"),
+    }
+}
+
+    // /// Validates whether is a valid something based on the boolean is_x smaller functions in this trait and returns a wrapped boolean. Example: check_u8(b"ACTG","is_basic_dna") returns a wrapped "true". Options for is_what are the names of the charset boolean functions:
+    // /// is_basic_dna, is_phred33, is_basic_rna,  
+    // /// The goal of this function would be to set is_what to a constant in the program, for example a program focused on illumina data might set a constant to phred33 and input as is_what, rather than having to call is_phred33 each time. This means we can easily make our program take phred33 or 64 by just changing the constant.
+    // fn check_u8(&self, is_what: &str) -> Result<bool, &str>;
+
+// /// Validates whether is a valid something based on the boolean is_x smaller functions in this trait and returns a wrapped boolean. Example: check_u8(b"ACTG","is_basic_dna") returns a wrapped "true". Options for is_what are the names of the charset boolean functions:
+    // /// "is_iupac_nucleotide", "is_iupac_amino_acid", "is_iupac",
+    // /// "is_phred33", "is_phred64", "is_solexa",  
+    // /// "is_basic_dna", "is_basic_rna", "is_basic_amino_acid",
+    // /// "is_homopolymer", "is_homopolymer_n", "is_homopolymer_not_n",
+    // /// "has_n", "has_gap",
+    // /// "is_ascii_letters", "is_ascii_letters_uppercase", "is_ascii_letters_lowercase" 
+    // /// The goal of this function would be to set is_what to a constant in the program, for example a program focused on dna data might set a constant to is_basic_dna and input as is_what rather than having to call is_basic_dna each time. Later, if we want to focus on rna, we can easily change our constant to is_basic_rna just by changing the constant.
+    // fn check_u8(&self, is_what: &str) -> Result<bool, &str>{
+    //     if validate_is_what(&is_what).unwrap() {
+    //         match is_what {
+    //             "is_phred33" => Ok(self.is_phred33()),
+    //             "is_phred64" => Ok(self.is_phred64()),
+    //             "is_solexa" => Ok(self.is_solexa()),
+    //             "is_iupac_nucleotide" => Ok(self.is_iupac_nucleotide()),
+    //             "is_iupac_amino_acid" => Ok(self.is_iupac_amino_acid()),
+    //             "is_iupac" => Ok(self.is_iupac()),
+    //             "is_basic_dna" => Ok(self.is_basic_dna()),
+    //             "is_basic_rna" => Ok(self.is_basic_rna()),
+    //             "is_basic_amino_acid" => Ok(self.is_basic_amino_acid()),
+    //             "is_homopolymer" => Ok(CheckU8::is_homopolymer(&self)),
+    //             "is_homopolymer_n" => Ok(self.is_homopolymer_n()),
+    //             "is_homopolymer_not_n" => Ok(self.is_homopolymer_not_n()),
+    //             "has_n" => Ok(self.has_n()),
+    //             "has_gap" => Ok(self.has_gap()),
+    //             "is_ascii_letters" => Ok(self.is_ascii_letters()),
+    //             "is_ascii_letters_uppercase" => Ok(self.is_ascii_letters_uppercase()),
+    //             "is_ascii_letters_lowercase" => Ok(self.is_ascii_letters_lowercase()),
+    //             _ => Err("Invalid is_what parameter, please choose a valid option")
+    //         }
+    //     } else {validate_is_what(&is_what)}
+    // }
+
+
+
 // /// Validates that 'is_what' parameter of the check_u8() function is a valid option.
 // pub fn validate_is_what<'a>(is_what: &str) -> Result<bool, &'a str> {
 //         match IS_WHAT_OPTIONS.contains(&is_what) {
